package	jp.co.sds_corp.transfer.source;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.StatusChangeEvent;
import jp.co.sds_corp.transfer.output.IRMIAdapter;
import jp.co.sds_corp.transfer.output.RMIAdapter;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* {@link RMIAdapter}からデータを受信します。
host,port,nameの各要素で、{@link RMIAdapter}の登録されているRMI registry
を指定します。
<PRE>
&lt;Source name="RMI" class="jp.co.sds_corp.transfer.source.RMISource"&gt;
	&lt;host&gt;localhost&lt;/host&gt;
	&lt;port&gt;1091&lt;/port&gt;
	&lt;name&gt;RMIAdapter&lt;/name&gt;
&lt;/Source&gt;
</PRE>
*/
public class RMISource extends AbstractSource
{
	private static final long serialVersionUID = 1L;
	String host;
	int port;
	String name;
	String url;
	Receiver rcv;
	IRMIAdapter remote;

	/**
	* 指定された初期情報でDummySourceを初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		rcv = new Receiver();

		host = config.get( "host" );
		port = Integer.valueOf( config.get( "port" ) ).intValue();
		name = config.get( "name" );

		url = "rmi://" + host + ":" + port + "/" + name;
		remote = (IRMIAdapter)Naming.lookup( url );
		remote.addReceiveListener( rcv );
	}

	/**
	* このサービスを開始します。
	* 裏Threadを起動し、指定された秒数間隔でダミーのデータを送信します。
	*/
	public void start() {
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		try {
			remote.removeReceiveListener( rcv );
		} catch ( Exception e ) {
			CmnLog.error( this, "removeReceiveListener() error", e );
		}
	}

	/**
	* RMI registry のホスト名(host)を取得します。
	* @return	hostの内容
	*/
	public String getHost() {
		return host;
	}
	/**
	* RMI registry のポート番号(port)を取得します。
	* @return	portの内容
	*/
	public int getPort() {
		return port;
	}
	/**
	* RMI registry の登録名(name)を取得します。
	* @return	nameの内容
	*/
	public String getName() {
		return name;
	}
	/**
	* RMI registry のURL(url)を取得します。
	* @return	urlの内容
	*/
	public String getUrl() {
		return url;
	}

	public class Receiver extends UnicastRemoteObject implements IRMISource
	{
		private static final long serialVersionUID = 1L;

		public Receiver() throws RemoteException, MalformedURLException {
		}

		public void received( ReceiveEvent e ) throws Exception
		{
			try {
				fireReceive( e.getData() );
			} catch ( Exception ex ) {
				CmnLog.error( this, "fireReceive() error", ex );
			}
		}
		public void statusChanged( StatusChangeEvent e ) throws Exception
		{
			try {
				fireStatusChange( e );
			} catch ( Exception ex ) {
				CmnLog.error( this, "fireStatusChange() error", ex );
			}
		}
	}

	/**
	* この取得元の文字列表現を返します。
	* @return	この取得元の文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += ":host=[" + host + "] port=[" + port + "]"
				+ " url=[" + url + "] name=[" + name + "]";
		return ret;
	}

}
